﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace PV178.Lecture2
{
    /// <summary>
    ///   Třída reprezentující hloupého prodejce, který prodává auta zadarmo :).
    /// </summary>
    internal class CarSeller
    {
        private const int DelayPaymentTresholdInSeconds = 2;

        private readonly ISet<Car> carsInStock = new HashSet<Car>();
        private readonly IDictionary<string, IList<SimpleInvoice>> invoices = new Dictionary<string, IList<SimpleInvoice>>();

        private long nextInvoiceId = 1;

        public IEnumerable<string> Customers
        {
            get { return invoices.Keys; }
        }


        /// <summary>
        ///   Prodá auto zakazníkovi a vystaví fakturu na prodané auto.
        /// </summary>
        /// <param name="car"> Prodané auto. </param>
        /// <param name="customer"> Nacionále zákazníka. </param>
        /// <returns> Faktura na prodej auta zákaníkovi. </returns>
        public SimpleInvoice Sell(Car car, string customer)
        {
            if (car == null)
                throw new ArgumentNullException("car");
            if (customer == null)
                throw new ArgumentNullException("customer");
            if (string.IsNullOrWhiteSpace(customer))
                throw new ArgumentException("Customer name can't be empty or whitespace.", "customer");
            if (!carsInStock.Contains(car))
                throw new InvalidOperationException("Can't sell car that's is not in stock.");

            if (!invoices.ContainsKey(customer))
                invoices.Add(customer, new List<SimpleInvoice>());

            var order = new SimpleInvoice(
                GetInvoiceID(),
                customer,
                car,
                this);
            invoices[customer].Add(order); // přístup ke slovníku pomocí 'indexeru'

            carsInStock.Remove(car);

            return order;
        }

        /// <summary>
        ///   Vypíše na konzoli seznam všech faktur u kterých se zpozdila platba nad stanovenou mez.
        /// </summary>
        public void ReportDelayedPayments()
        {
            DateTime now = DateTime.Now;

            // LINQ dotaz vracé rozhraní IEnumerable<T>, kde T je typ určený klazulí select: 
            //   v tomto případě se jedná o anonymní typ se dvěma vlastnostmi: string Customer a int Delay.
            var delayedPayments = from s in invoices
                                  from i in s.Value
                                  where !i.PaymentDate.HasValue && (now - i.InvoiceDate).Seconds > DelayPaymentTresholdInSeconds
                                  select new
                                             {
                                                 i.Customer,
                                                 Delay = (now - i.InvoiceDate).Seconds - DelayPaymentTresholdInSeconds
                                             };

            Console.WriteLine("Dluznici:");
            foreach (var item in delayedPayments)
            {
                Console.WriteLine("{0}, {1} s", item.Customer, item.Delay);
            }
        }

        /// <summary>
        ///   Vloží auto na sklad.
        /// </summary>
        /// <param name="car"> Nově pořízené auto. </param>
        public bool Stock(Car car)
        {
            return carsInStock.Add(car);
        }


        /// <summary>
        ///   Získá další pořadové číslo pro fakturu.
        /// </summary>
        /// <returns> </returns>
        protected long GetInvoiceID()
        {
            return nextInvoiceId++;
        }
    }
}