﻿using System;
using System.Collections.Generic;

namespace PV178.Lecture2
{
    internal class SimpleInvoice
    {
        private static readonly Dictionary<InvoiceState, string> InvoiceStatusTranslationMatrix = new Dictionary<InvoiceState, string>
                                                                                                      {
                                                                                                          {InvoiceState.Active, "otevrena"},
                                                                                                          {InvoiceState.Paid, "zaplacena"},
                                                                                                          {InvoiceState.Canceled, "stornovana"}
                                                                                                      };

        /// <summary>
        ///   Jedinečný identifikátor faktury.
        /// </summary>
        public long ID { get; set; }

        /// <summary>
        ///   Datum založení faktury.
        /// </summary>
        public DateTime InvoiceDate { get; private set; }

        /// <summary>
        ///   Datum zaplacení faktury.
        /// </summary>
        public DateTime? PaymentDate { get; private set; }

        /// <summary>
        ///   Vrací fakturované auto.
        /// </summary>
        public Car Car { get; private set; }

        /// <summary>
        ///   Vrací prodejce, který fakturu vystavil.
        /// </summary>
        public CarSeller Seller { get; private set; }

        /// <summary>
        ///   Vrací jméno zákazníka.
        /// </summary>
        public string Customer { get; private set; }

        /// <summary>
        ///   Vrací stav faktury.
        /// </summary>
        public InvoiceState State { get; private set; }


        protected bool CanBeChanged
        {
            get { return State == InvoiceState.Active; }
        }

        protected bool CanBeCancelled
        {
            get { return State == InvoiceState.Active; }
        }

        /// <summary>
        ///   Vytvoří fakturu pro zadaného zákazníka.
        /// </summary>
        /// <param name="id"> Identifikační číslo faktury. </param>
        /// <param name="customer"> Nacionále zákazníka. </param>
        /// <param name="car"> Prodávané auto. </param>
        /// <param name="seller"> Prodejce. </param>
        public SimpleInvoice(long id, string customer, Car car, CarSeller seller)
        {
            ID = id;
            Customer = customer;
            Car = car;
            Seller = seller;
            InvoiceDate = DateTime.Now;
        }

        /// <summary>
        ///   Nastaví fakturu jako zaplacenou.
        /// </summary>
        /// <param name="date"> Datum zaplacení. </param>
        /// <exception cref="InvalidOperationException">Fakturu už nelze změnit.</exception>
        public void MarkAsPayed(DateTime date)
        {
            if (!CanBeChanged)
                throw new InvalidOperationException("Invoice can't be changed.");

            PaymentDate = date;
            State = InvoiceState.Paid;
        }

        /// <summary>
        ///   Stornuje fakturu.
        /// </summary>
        /// <exception cref="InvalidOperationException">Faktura už nemůže být stornována.</exception>
        public void Cancel()
        {
            if (!CanBeCancelled)
                throw new InvalidOperationException("Invoice can't be cancelled.");

            State = InvoiceState.Canceled;
        }


        // zde následují metody a operátory pro porovnání a ToString()

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != typeof (SimpleInvoice)) return false;

            SimpleInvoice other = (SimpleInvoice) obj;
            return other.ID == ID;
        }

        public override int GetHashCode()
        {
            return ID.GetHashCode();
        }

        public static bool operator ==(SimpleInvoice left, SimpleInvoice right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(SimpleInvoice left, SimpleInvoice right)
        {
            return !Equals(left, right);
        }


        public override string ToString()
        {
            string paymentInformationText = GetDisplayTextForPayment(PaymentDate);

            return string.Format("Faktura {0:D6}, {4}, zakaznik {1}, den vystaveni {2}, {3}",
                                 ID, Customer, InvoiceDate, paymentInformationText, GetDisplayTextForInvoiceStatus(State));
        }

        private static string GetDisplayTextForInvoiceStatus(InvoiceState state)
        {
            return InvoiceStatusTranslationMatrix[state];
        }

        private static string GetDisplayTextForPayment(DateTime? payed)
        {
            string paymentInformationText;
            if (payed == null)
            {
                paymentInformationText = "nezaplaceno";
            }
            else
            {
                paymentInformationText = "zaplaceno " + payed.Value;
            }
            return paymentInformationText;
        }
    }
}