﻿using System;

namespace PV178.Lecture2
{
    /// <summary>
    ///   Struktura představující jedinečný identifikátor automobilu.
    /// </summary>
    public struct VehicleIdentificationNumber
    {
        /// <summary>
        ///   Prototyp prázného VINu - struktury bez dat. Používá se pro otestování, jestli má struktura počáteční hodnotu. Tento prototyp je sice veřejná datová položka, ale statická a neměnná (readonly).
        /// </summary>
        /// <remarks>
        ///   V tomto případě nelze vytvořit konstantu a datová položka s modifikátory readonly static je přirozenou náhradou.
        /// </remarks>
        public static readonly VehicleIdentificationNumber Empty;


        /// <summary>
        ///   Seriové číslo v rámci daného výrobce.
        /// </summary>
        public int UniqueId { get; private set; }

        /// <summary>
        ///   Identifikátor výrobce.
        /// </summary>
        public string Manufacturer { get; private set; }


        /// <summary>
        ///   Konstruktor pro vytvoření výrobce.
        /// </summary>
        /// <param name="manufacturer"> </param>
        /// <param name="uniqueId"> </param>
        public VehicleIdentificationNumber(string manufacturer, int uniqueId) : this()
        {
            if (manufacturer == null)
                throw new ArgumentNullException("manufacturer");
            if (string.IsNullOrWhiteSpace(manufacturer))
                throw new ArgumentException("Invalid manufacturer identifier.", "manufacturer");
            if (uniqueId <= 0)
                throw new ArgumentOutOfRangeException("uniqueId", uniqueId, "ID must be greater than 0.");

            Manufacturer = manufacturer;
            UniqueId = uniqueId;
        }


        public override string ToString()
        {
            return Manufacturer + UniqueId;
        }


        // Deklarace vlastního operátoru rovnosti (==) pro větší pohodlí při práci s typem.
        // Vlastní Equals předefinovat nemusíme, protože porovnání struktur probíhá implicitně 
        // podle jejich obsahu a to nám vyhovuje.

        public static bool operator ==(VehicleIdentificationNumber left, VehicleIdentificationNumber right)
        {
            return left.Equals(right);
        }

        public static bool operator !=(VehicleIdentificationNumber left, VehicleIdentificationNumber right)
        {
            return !(left == right);
        }
    }
}